/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.servlet.fusion;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.util.HashMap;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xquark.mediator.Mediator;
import org.xquark.serialize.XMLSerializer;
import org.xquark.xml.xdbc.*;

public class XQFusionServlet extends HttpServlet {

	private static final String JNDI_SOURCE = "java:comp/env/jdbc/TestDB";
	private Mediator mediator;
    private SAXParserFactory factory;
    private HashMap mappings;
    private ServletContext context = null;

    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        context = config.getServletContext();
        factory = SAXParserFactory.newInstance();
        factory.setNamespaceAware(true);
        File configFile = new File(config.getServletContext().getRealPath("WEB-INF/queries/main.xml"));
		try {
            mediator = new Mediator(configFile.toURL().toString());
        } catch (MalformedURLException e) {
 			throw new ServletException("Could not create mediator", e);
        } catch (XMLDBCException e) {
			throw new ServletException("Could not create mediator", e);
        }
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException 
    {
    	long t0 = System.currentTimeMillis();
        PrintWriter out = response.getWriter();
		XMLConnection conn = null;
        try {
            String query = request.getParameter("exec");
            if (query != null) {
            	conn = mediator.getConnection();
                execQuery(conn, query, out);
            }
            else {
                reportError(
                    response,
                    "Unrecognized command: <B>"
                        + request.getContextPath()
                        + request.getServletPath()
                        + request.getQueryString()
                        + "</B>.",
                    "query.jsp");
                return;
            }
        }
        catch (XMLDBCException e) {
            reportException(response, e, "query.jsp");
        }
        finally {
            out.close();
            if (conn != null) 
            try {
            	conn.close();
            } catch (XMLDBCException e) {
				// ignore
            }
            context.log("Request processed in "+(System.currentTimeMillis()-t0)+" ms.");
        }
    }

    private void execQuery(XMLConnection conn, String query, PrintWriter out)
        throws ServletException, XMLDBCException {
        PreparedXMLStatement stmt = null;
        XMLResultSet xrs = null;
        try {
            stmt = conn.prepareStatement(query);
            xrs = stmt.executeQuery();
            XMLSerializer serializer = new XMLSerializer(out, "iso-8859-1");
            xrs.setContentHandler(serializer);
            xrs.setLexicalHandler(serializer);
            serializer.startDocument();
            //serializer.processingInstruction("xml-stylesheet", "href=\"xsl/beautify.xsl\" type=\"text/xsl\"");
            serializer.startElement(
                "",
                "results",
                "results",
                new AttributesImpl());
            while (xrs.hasNext())
                xrs.nextAsSAX();
            serializer.endElement("", "results", "results");
            serializer.endDocument();
        } catch (SAXException e) {
			throw new ServletException("Error while producing document result", e);
        }
        finally {
            if (xrs != null)
                xrs.close();
            if (stmt != null)
                stmt.close();
        }
    }

    private void reportMessage(
        HttpServletResponse response,
        String message,
        String url)
        throws IOException {
        response.sendRedirect(
            response.encodeRedirectURL(
                "info.jsp?url=" + url + "&message=" + message));
    }

    private void reportError(
        HttpServletResponse response,
        String error,
        String url)
        throws IOException {
        response.sendRedirect(
            response.encodeRedirectURL(
                "error.jsp?url=" + url + "&message=" + error));
    }

    private void reportException(
        HttpServletResponse response,
        Exception e,
        String url)
        throws IOException {
        /*
        StringWriter sw = new StringWriter();
        e.printStackTrace(new PrintWriter(sw));
        String message = "<pre>"+sw.toString()+"</pre>";
        response.sendRedirect(response.encodeRedirectURL("exception.jsp?url="+url+"&message="+message));
        */
        PrintWriter out = response.getWriter();
        out.println("<html><body><pre>");
        e.printStackTrace(out);
        out.println("</pre></body></html>");
    }
}
