module namespace ns = "http://www.xquark.org/XQuery/Samples/viewDefs";
import schema namespace bib = "http://www.xquark.org/XQuery/use-cases/bib" at "bib.xsd";
import schema namespace r = "http://www.xquark.org/XQuery/use-cases/reviews" at "reviews.xsd";
import schema namespace p = "http://www.xquark.org/XQuery/use-cases/prices" at "prices.xsd";

declare function ns:bib() as element(bib:bib) {
	<bib:bib>
		{
			for $b in collection("book")/*:book
			order by $b/id
			return 
				<book year="{$b/year}">
					{ $b/title }
					{
						for $a in collection("author")/*:author,
							$p in collection("person")/*:person
						where $a/bookid = $b/id and $a/personid = $p/id
						order by $a/rank
						return
							<author>
								{ $p/last }
								{ $p/first }
							</author>
					}
					{
						for $e in collection("editor")/*:editor,
							$p in collection("person")/*:person
						where $e/bookid = $b/id and $e/personid = $p/id
						order by $e/rank
						return
							<editor>
								{ $p/last }
								{ $p/first }
								{ $p/affiliation }
							</editor>
					}
					{
						for $pub in collection("publisher")/*:publisher
						where $pub/id = $b/pubid
						return
							<publisher>{ data($pub/name) }</publisher>
					}
					{ 
						for $price in collection("price")/*:price
						where $price/source = "www.bn.com" and $b/id = $price/bookid
						return
							<price>{ data($price/value) }</price> 
					}
				</book>
		}
	</bib:bib>
};

declare function ns:reviews() as element(r:reviews) {
	<r:reviews>
		{
			for $e in collection("review")/*:review
			order by $e/id
			return 
				<entry>
					{
						for $b in collection("book")/*:book
						where $e/bookid = $b/id
						return 
							$b/title
					}
					{ 
						for $price in collection("price")/*:price
						where $price/source = "www.amazon.com" and $e/bookid = $price/bookid
						return
							<price>{ data($price/value) }</price> 
					}
					<review>
						{ data($e/comment) }
					</review>
				</entry>
		}
	</r:reviews>
};

declare function ns:prices() as element(p:prices) {
	<p:prices>
		{
			for $p in collection("price")/*:price,
				$b in collection("book")/*:book
			where $p/bookid = $b/id
			order by $p/id
			return 
				<book>
					{ $b/title }
					{ $p/source }
					<price>{ data($p/value) }</price>
				</book>
		}
	</p:prices>
}
